@@ -57,4 +57,7 @@ def qgis_server_version() -> str:
             f"Cannot check version with PyQGIS, check your QGIS installation or your PYTHONPATH"
             f"{Level.End}"
         )
+        import os
+        print(f"Current user : {os.environ.get('USER')}")
+        print(f'PYTHONPATH={os.getenv("PYTHONPATH")}')
     return ''
